@@ -54,11 +54,13 @@ namespace gr {
       d_pc_rpc_set(false),
       d_update_rate(false),
       d_max_output_buffer(std::max(output_signature->max_streams(),1), -1),
-      d_min_output_buffer(std::max(output_signature->max_streams(),1), -1)
+      d_min_output_buffer(std::max(output_signature->max_streams(),1), -1),
+      d_pmt_done(pmt::intern("done")),
+      d_system_port(pmt::intern("system"))
   {
     global_block_registry.register_primitive(alias(), this);
-    message_port_register_in(pmt::mp("system"));
-    set_msg_handler(pmt::mp("system"), boost::bind(&block::system_handler, this, _1));
+    message_port_register_in(d_system_port);
+    set_msg_handler(d_system_port, boost::bind(&block::system_handler, this, _1));
 
     configure_default_loggers(d_logger, d_debug_logger, symbol_name());
   }
@@ -706,7 +708,7 @@ namespace gr {
   {
     //std::cout << "system_handler " << msg << "\n";
     pmt::pmt_t op = pmt::car(msg);
-    if(pmt::eqv(op, pmt::mp("done"))){
+    if(pmt::eqv(op, d_pmt_done)){
         d_finished = pmt::to_long(pmt::cdr(msg));
         global_block_registry.notify_blk(alias());
     } else {
@@ -733,16 +735,10 @@ namespace gr {
         pmt::pmt_t target = pmt::car(currlist);
 
         pmt::pmt_t block = pmt::car(target);
-        pmt::pmt_t port = pmt::mp("system");
 
         currlist = pmt::cdr(currlist);
         basic_block_sptr blk = global_block_registry.block_lookup(block);
-        blk->post(port, pmt::cons(pmt::mp("done"), pmt::mp(true)));
-
-        //std::cout << "notify finished --> ";
-        //pmt::print(pmt::cons(block,port));
-        //std::cout << "\n";
-
+        blk->post(d_system_port, pmt::cons(d_pmt_done, pmt::mp(true)));
         }
     }
   }
@@ -839,6 +835,13 @@ namespace gr {
         "", "Total clock cycles in calls to work", RPC_PRIVLVL_MIN,
         DISPTIME | DISPOPTSTRIP)));
 
+    d_rpc_vars.push_back(
+      rpcbasic_sptr(new rpcbasic_register_get<block, float>(
+        alias(), "avg throughput", &block::pc_throughput_avg,
+        pmt::mp(0), pmt::mp(1e9), pmt::mp(0),
+        "items/s", "Average items throughput in call to work", RPC_PRIVLVL_MIN,
+        DISPTIME | DISPOPTSTRIP)));
+
     d_rpc_vars.push_back(
       rpcbasic_sptr(new rpcbasic_register_get<block, std::vector<float> >(
         alias(), "input \% full", &block::pc_input_buffers_full,
