@@ -297,7 +297,7 @@ serialize(pmt_t obj, std::streambuf &sb)
     }
 
     if(is_real(obj)) {
-      float i = to_double(obj);
+      double i = to_double(obj);
       ok = serialize_untagged_u8(PST_DOUBLE, sb);
       ok &= serialize_untagged_f64(i, sb);
       return ok;
@@ -531,7 +531,7 @@ deserialize(std::streambuf &sb)
   uint32_t	u32;
   uint64_t	u64;
   double	f64;
-  static char   tmpbuf[1024];
+  char          tmpbuf[1024];
 
   if (!deserialize_untagged_u8(&tag, sb))
     return PMT_EOF;
