@@ -252,7 +252,7 @@ namespace gr {
       d_buffer = new char[d_buffer_size_bytes];
 
       if(CHATTY_DEBUG) {
-        GR_LOG_DEBUG(d_logger, boost::format("[%1%]: sample resolution = %d bits") \
+        GR_LOG_DEBUG(d_logger, boost::format("[%1%]: sample resolution = %2% bits") \
                      % snd_pcm_name(d_pcm_handle)                       \
                      % snd_pcm_hw_params_get_sbits(d_hw_params));
       }
@@ -310,7 +310,7 @@ namespace gr {
                           gr_vector_const_void_star &input_items,
                           gr_vector_void_star &output_items)
     {
-      typedef gr_int16 sample_t; // the type of samples we're creating
+      typedef int16_t sample_t; // the type of samples we're creating
       static const float scale_factor = 1.0 / std::pow(2.0f, 16-1);
 
       unsigned int nchan = output_items.size ();
@@ -348,7 +348,7 @@ namespace gr {
                               gr_vector_const_void_star &input_items,
                               gr_vector_void_star &output_items)
     {
-      typedef gr_int16 sample_t; // the type of samples we're creating
+      typedef int16_t sample_t; // the type of samples we're creating
       static const float scale_factor = 1.0 / std::pow(2.0f, 16-1);
 
       float **out = (float**)&output_items[0];
@@ -385,7 +385,7 @@ namespace gr {
                           gr_vector_const_void_star &input_items,
                           gr_vector_void_star &output_items)
     {
-      typedef gr_int32 sample_t; // the type of samples we're creating
+      typedef int32_t sample_t; // the type of samples we're creating
       static const float scale_factor = 1.0 / std::pow(2.0f, 32-1);
 
       unsigned int nchan = output_items.size ();
@@ -423,7 +423,7 @@ namespace gr {
                               gr_vector_const_void_star &input_items,
                               gr_vector_void_star &output_items)
     {
-      typedef gr_int32 sample_t; // the type of samples we're creating
+      typedef int32_t sample_t; // the type of samples we're creating
       static const float scale_factor = 1.0 / std::pow(2.0f, 32-1);
 
       float **out = (float**)&output_items[0];
@@ -472,7 +472,7 @@ namespace gr {
           }
           continue;  // try again
         }
-
+#ifdef ESTRPIPE
         else if(r == -ESTRPIPE) {   // h/w is suspended (whatever that means)
                                     // This is apparently related to power management
           d_nsuspends++;
@@ -482,7 +482,7 @@ namespace gr {
           }
           continue;   // try again
         }
-
+#endif
         else if(r < 0) {
           output_error_msg("snd_pcm_readi failed", r);
           return false;
