@@ -85,6 +85,11 @@ public:
     d_unitType = type;
   }
 
+  std::string unitType()
+  {
+    return d_unitType;
+  }
+
   void setYUnitType(const std::string &type)
   {
     d_yUnitType = type;
@@ -130,6 +135,10 @@ TimeDomainDisplayPlot::TimeDomainDisplayPlot(int nplots, QWidget* parent)
   d_xdata = new double[d_numPoints];
   memset(d_xdata, 0x0, d_numPoints*sizeof(double));
 
+  d_tag_text_color = Qt::black;
+  d_tag_background_color = Qt::white;
+  d_tag_background_style = Qt::NoBrush;
+
   d_zoomer = new TimeDomainDisplayZoomer(canvas(), 0);
 
 #if QWT_VERSION < 0x060000
@@ -147,6 +156,7 @@ TimeDomainDisplayPlot::TimeDomainDisplayPlot(int nplots, QWidget* parent)
 
   d_semilogx = false;
   d_semilogy = false;
+  d_autoscale_shot = false;
 
   setAxisScaleEngine(QwtPlot::xBottom, new QwtLinearScaleEngine);
   setXaxis(0, d_numPoints);
@@ -158,10 +168,16 @@ TimeDomainDisplayPlot::TimeDomainDisplayPlot(int nplots, QWidget* parent)
 
   QList<QColor> colors;
   colors << QColor(Qt::blue) << QColor(Qt::red) << QColor(Qt::green)
+	 << QColor(Qt::black) << QColor(Qt::cyan) << QColor(Qt::magenta)
+	 << QColor(Qt::yellow) << QColor(Qt::gray) << QColor(Qt::darkRed)
+	 << QColor(Qt::darkGreen) << QColor(Qt::darkBlue) << QColor(Qt::darkGray)
+	 // cycle through all colors again to increase time_sink_f input limit
+	 // from 12 to 24, otherwise you get a segfault
+	 << QColor(Qt::blue) << QColor(Qt::red) << QColor(Qt::green)
 	 << QColor(Qt::black) << QColor(Qt::cyan) << QColor(Qt::magenta)
 	 << QColor(Qt::yellow) << QColor(Qt::gray) << QColor(Qt::darkRed)
 	 << QColor(Qt::darkGreen) << QColor(Qt::darkBlue) << QColor(Qt::darkGray);
-
+	 
   // Setup dataPoints and plot vectors
   // Automatically deleted when parent is deleted
   for(int i = 0; i < d_nplots; i++) {
@@ -190,8 +206,23 @@ TimeDomainDisplayPlot::TimeDomainDisplayPlot(int nplots, QWidget* parent)
 
   d_tag_markers.resize(d_nplots);
   d_tag_markers_en = std::vector<bool>(d_nplots, true);
+
+  d_trigger_lines[0] = new QwtPlotMarker();
+  d_trigger_lines[0]->setLineStyle(QwtPlotMarker::HLine);
+  d_trigger_lines[0]->setLinePen(QPen(Qt::red, 0.6, Qt::DashLine));
+  d_trigger_lines[0]->setRenderHint(QwtPlotItem::RenderAntialiased);
+  d_trigger_lines[0]->setXValue(0.0);
+  d_trigger_lines[0]->setYValue(0.0);
+
+  d_trigger_lines[1] = new QwtPlotMarker();
+  d_trigger_lines[1]->setLineStyle(QwtPlotMarker::VLine);
+  d_trigger_lines[1]->setLinePen(QPen(Qt::red, 0.6, Qt::DashLine));
+  d_trigger_lines[1]->setRenderHint(QwtPlotItem::RenderAntialiased);
+  d_trigger_lines[1]->setXValue(0.0);
+  d_trigger_lines[1]->setYValue(0.0);
 }
 
+
 TimeDomainDisplayPlot::~TimeDomainDisplayPlot()
 {
   for(int i = 0; i < d_nplots; i++)
@@ -318,9 +349,7 @@ TimeDomainDisplayPlot::plotNewData(const std::vector<double*> dataPoints,
               m->setXValue(sample_offset);
               m->setYValue(yval);
 
-              QBrush brush;
-              brush.setColor(QColor(0xC8, 0x2F, 0x1F));
-              brush.setStyle(Qt::SolidPattern);
+              QBrush brush(getTagBackgroundColor(), getTagBackgroundStyle());
 
               QPen pen;
               pen.setColor(Qt::black);
@@ -342,8 +371,10 @@ TimeDomainDisplayPlot::plotNewData(const std::vector<double*> dataPoints,
 #else
               m->setSymbol(sym);
 #endif
+              QwtText tag_label(s.str().c_str());
+              tag_label.setColor(getTagTextColor());
+              m->setLabel(tag_label);
 
-              m->setLabel(QwtText(s.str().c_str()));
               m->attach(this);
 
               if(!(show && d_tag_markers_en[which])) {
@@ -360,7 +391,14 @@ TimeDomainDisplayPlot::plotNewData(const std::vector<double*> dataPoints,
               QString orig = (*mitr)->label().text();
               s << std::endl;
               orig.prepend(s.str().c_str());
-              (*mitr)->setLabel(orig);
+
+              QwtText newtext(orig);
+              newtext.setColor(getTagTextColor());
+
+              QBrush brush(getTagBackgroundColor(), getTagBackgroundStyle());
+              newtext.setBackgroundBrush(brush);
+
+              (*mitr)->setLabel(newtext);
             }
           }
 
@@ -381,6 +419,10 @@ TimeDomainDisplayPlot::plotNewData(const std::vector<double*> dataPoints,
 	  }
 	}
 	_autoScale(bottom, top);
+        if(d_autoscale_shot) {
+          d_autoscale_state = false;
+          d_autoscale_shot = false;
+        }
       }
 
       replot();
@@ -469,12 +511,21 @@ TimeDomainDisplayPlot::setAutoScale(bool state)
   d_autoscale_state = state;
 }
 
+void
+TimeDomainDisplayPlot::setAutoScaleShot()
+{
+  d_autoscale_state = true;
+  d_autoscale_shot = true;
+}
+
+
 void
 TimeDomainDisplayPlot::setSampleRate(double sr, double units,
 				     const std::string &strunits)
 {
   double newsr = sr/units;
-  if(newsr != d_sample_rate) {
+  if((newsr != d_sample_rate) ||
+     (((TimeDomainDisplayZoomer*)d_zoomer)->unitType() != strunits)) {
     d_sample_rate = sr/units;
     _resetXAxisPoints();
 
@@ -488,6 +539,12 @@ TimeDomainDisplayPlot::setSampleRate(double sr, double units,
   }
 }
 
+double
+TimeDomainDisplayPlot::sampleRate() const
+{
+  return d_sample_rate;
+}
+
 void
 TimeDomainDisplayPlot::stemPlot(bool en)
 {
@@ -558,6 +615,43 @@ TimeDomainDisplayPlot::enableTagMarker(int which, bool en)
     throw std::runtime_error("TimeDomainDisplayPlot: enabled tag marker does not exist.\n");
 }
 
+const QColor
+TimeDomainDisplayPlot::getTagTextColor()
+{
+  return d_tag_text_color;
+}
+
+const QColor
+TimeDomainDisplayPlot::getTagBackgroundColor()
+{
+  return d_tag_background_color;
+}
+
+const Qt::BrushStyle
+TimeDomainDisplayPlot::getTagBackgroundStyle()
+{
+  return d_tag_background_style;
+}
+
+void
+TimeDomainDisplayPlot::setTagTextColor(QColor c)
+{
+  d_tag_text_color = c;
+}
+
+void
+TimeDomainDisplayPlot::setTagBackgroundColor(QColor c)
+{
+  d_tag_background_color = c;
+}
+
+void
+TimeDomainDisplayPlot::setTagBackgroundStyle(Qt::BrushStyle b)
+{
+  d_tag_background_style = b;
+}
+
+
 void
 TimeDomainDisplayPlot::setYLabel(const std::string &label,
                                  const std::string &unit)
@@ -569,4 +663,26 @@ TimeDomainDisplayPlot::setYLabel(const std::string &label,
   ((TimeDomainDisplayZoomer*)d_zoomer)->setYUnitType(unit);
 }
 
+void
+TimeDomainDisplayPlot::attachTriggerLines(bool en)
+{
+  if(en) {
+    d_trigger_lines[0]->attach(this);
+    d_trigger_lines[1]->attach(this);
+  }
+  else {
+    d_trigger_lines[0]->detach();
+    d_trigger_lines[1]->detach();
+  }
+}
+
+void
+TimeDomainDisplayPlot::setTriggerLines(double x, double y)
+{
+  d_trigger_lines[0]->setXValue(x);
+  d_trigger_lines[0]->setYValue(y);
+  d_trigger_lines[1]->setXValue(x);
+  d_trigger_lines[1]->setYValue(y);
+}
+
 #endif /* TIME_DOMAIN_DISPLAY_PLOT_C */
