@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2012 Free Software Foundation, Inc.
+ * Copyright 2012,2015 Free Software Foundation, Inc.
  *
  * This file is part of GNU Radio
  *
@@ -47,12 +47,15 @@ namespace gr {
       std::string d_name;
       int d_nconnections;
 
+      const pmt::pmt_t d_port;
+
       bool d_shift;
       fft::fft_complex *d_fft;
 
       int d_index;
       std::vector<gr_complex*> d_residbufs;
       std::vector<double*> d_magbufs;
+      double* d_pdu_magbuf;
       float *d_fbuf;
       float *d_tmpbuf;
 
@@ -74,6 +77,9 @@ namespace gr {
       // The message is a PMT pair (intern('freq'), double(frequency)).
       void handle_set_freq(pmt::pmt_t msg);
 
+      // Handles message input port for displaying PDU samples.
+      void handle_pdus(pmt::pmt_t msg);
+
       // Members used for triggering scope
       trigger_mode d_trigger_mode;
       float d_trigger_level;
@@ -119,6 +125,7 @@ namespace gr {
       void set_update_time(double t);
 
       void set_title(const std::string &title);
+      void set_y_label(const std::string &label, const std::string &unit);
       void set_line_label(int which, const std::string &label);
       void set_line_color(int which, const std::string &color);
       void set_line_width(int which, int width);
@@ -149,6 +156,7 @@ namespace gr {
       void clear_min_hold();
       void disable_legend();
       void reset();
+      void enable_axis_labels(bool en);
 
       int work(int noutput_items,
 	       gr_vector_const_void_star &input_items,
