@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2012,2014 Free Software Foundation, Inc.
+ * Copyright 2012,2014-2015 Free Software Foundation, Inc.
  *
  * This file is part of GNU Radio
  *
@@ -54,12 +54,13 @@ namespace gr {
 				       int nconnections,
 				       QWidget *parent)
       : sync_block("freq_sink_f",
-                   io_signature::make(nconnections, nconnections, sizeof(float)),
+                   io_signature::make(0, nconnections, sizeof(float)),
                    io_signature::make(0, 0, 0)),
 	d_fftsize(fftsize), d_fftavg(1.0),
 	d_wintype((filter::firdes::win_type)(wintype)),
 	d_center_freq(fc), d_bandwidth(bw), d_name(name),
-	d_nconnections(nconnections), d_parent(parent)
+  d_nconnections(nconnections), d_parent(parent),
+  d_port(pmt::mp("freq"))
     {
       // Required now for Qt; argc must be greater than 0 and argv
       // must have at least one valid character. Must be valid through
@@ -71,11 +72,16 @@ namespace gr {
 
       // setup output message port to post frequency when display is
       // double-clicked
-      message_port_register_out(pmt::mp("freq"));
-      message_port_register_in(pmt::mp("freq"));
-      set_msg_handler(pmt::mp("freq"),
+      message_port_register_out(d_port);
+      message_port_register_in(d_port);
+      set_msg_handler(d_port,
                       boost::bind(&freq_sink_f_impl::handle_set_freq, this, _1));
 
+      // setup PDU handling input port
+      message_port_register_in(pmt::mp("in"));
+      set_msg_handler(pmt::mp("in"),
+                      boost::bind(&freq_sink_f_impl::handle_pdus, this, _1));
+
       d_main_gui = NULL;
 
       // Perform fftshift operation;
@@ -92,6 +98,7 @@ namespace gr {
                                      volk_get_alignment());
 
       d_index = 0;
+      // save the last "connection" for the PDU memory
       for(int i = 0; i < d_nconnections; i++) {
 	d_residbufs.push_back((float*)volk_malloc(d_fftsize*sizeof(float),
                                                   volk_get_alignment()));
@@ -102,6 +109,14 @@ namespace gr {
 	memset(d_magbufs[i], 0, d_fftsize*sizeof(double));
       }
 
+      d_residbufs.push_back((float*)volk_malloc(d_fftsize*sizeof(float),
+                                                volk_get_alignment()));
+      d_pdu_magbuf = (double*)volk_malloc(d_fftsize*sizeof(double),
+                                          volk_get_alignment());
+      d_magbufs.push_back(d_pdu_magbuf);
+      memset(d_residbufs[d_nconnections], 0, d_fftsize*sizeof(float));
+      memset(d_pdu_magbuf, 0, d_fftsize*sizeof(double));
+
       buildwindow();
 
       initialize();
@@ -114,7 +129,8 @@ namespace gr {
       if(!d_main_gui->isClosed())
         d_main_gui->close();
 
-      for(int i = 0; i < d_nconnections; i++) {
+      // +1 to handle PDU buffers; will also take care of d_pdu_magbuf
+      for(int i = 0; i < d_nconnections+1; i++) {
 	volk_free(d_residbufs[i]);
 	volk_free(d_magbufs[i]);
       }
@@ -146,13 +162,10 @@ namespace gr {
       }
 
       // If a style sheet is set in the prefs file, enable it here.
-      std::string qssfile = prefs::singleton()->get_string("qtgui","qss","");
-      if(qssfile.size() > 0) {
-        QString sstext = get_qt_style_sheet(QString(qssfile.c_str()));
-        d_qApplication->setStyleSheet(sstext);
-      }
+      check_set_qss(d_qApplication);
 
-      d_main_gui = new FreqDisplayForm(d_nconnections, d_parent);
+      int numplots = (d_nconnections > 0) ? d_nconnections : 1;
+      d_main_gui = new FreqDisplayForm(numplots, d_parent);
       set_fft_window(d_wintype);
       set_fft_size(d_fftsize);
       set_frequency_range(d_center_freq, d_bandwidth);
@@ -248,6 +261,13 @@ namespace gr {
       d_main_gui->setYaxis(min, max);
     }
 
+    void
+    freq_sink_f_impl::set_y_label(const std::string &label,
+                                  const std::string &unit)
+    {
+        d_main_gui->setYLabel(label, unit);
+    }
+
     void
     freq_sink_f_impl::set_update_time(double t)
     {
@@ -395,6 +415,12 @@ namespace gr {
       d_main_gui->autoScale(en);
     }
 
+    void
+    freq_sink_f_impl::enable_axis_labels(bool en)
+    {
+        d_main_gui->setAxisLabels(en);
+    }
+
     void
     freq_sink_f_impl::enable_control_panel(bool en)
     {
@@ -512,7 +538,8 @@ namespace gr {
 
       if(newfftsize != d_fftsize) {
 	// Resize residbuf and replace data
-	for(int i = 0; i < d_nconnections; i++) {
+        // +1 to handle PDU buffers
+	for(int i = 0; i < d_nconnections+1; i++) {
 	  volk_free(d_residbufs[i]);
 	  volk_free(d_magbufs[i]);
 
@@ -525,6 +552,9 @@ namespace gr {
 	  memset(d_magbufs[i], 0, newfftsize*sizeof(double));
 	}
 
+        // Update the pointer to the newly allocated memory
+        d_pdu_magbuf = d_magbufs[d_nconnections];
+
 	// Set new fft size and reset buffer index
 	// (throws away any currently held data, but who cares?)
 	d_fftsize = newfftsize;
@@ -561,8 +591,8 @@ namespace gr {
     {
       if(d_main_gui->checkClicked()) {
         double freq = d_main_gui->getClickedFreq();
-        message_port_pub(pmt::mp("freq"),
-                         pmt::cons(pmt::mp("freq"),
+        message_port_pub(d_port,
+                         pmt::cons(d_port,
                                    pmt::from_double(freq)));
       }
     }
@@ -697,5 +727,87 @@ namespace gr {
       return noutput_items;
     }
 
+    void
+    freq_sink_f_impl::handle_pdus(pmt::pmt_t msg)
+    {
+      size_t len;
+      pmt::pmt_t dict, samples;
+
+      // Test to make sure this is either a PDU or a uniform vector of
+      // samples. Get the samples PMT and the dictionary if it's a PDU.
+      // If not, we throw an error and exit.
+      if(pmt::is_pair(msg)) {
+        dict = pmt::car(msg);
+        samples = pmt::cdr(msg);
+      }
+      else if(pmt::is_uniform_vector(msg)) {
+        samples = msg;
+      }
+      else {
+        throw std::runtime_error("time_sink_c: message must be either "
+                                 "a PDU or a uniform vector of samples.");
+      }
+
+      len = pmt::length(samples);
+
+      const float *in;
+      if(pmt::is_f32vector(samples)) {
+        in = (const float*)pmt::f32vector_elements(samples, len);
+      }
+      else {
+        throw std::runtime_error("freq_sink_f: unknown data type "
+                                 "of samples; must be float.");
+      }
+
+      // Plot if we're past the last update time
+      if(gr::high_res_timer_now() - d_last_time > d_update_time) {
+        d_last_time = gr::high_res_timer_now();
+
+        // Update the FFT size from the application
+        fftresize();
+        windowreset();
+        check_clicked();
+
+        int winoverlap = 4;
+        int fftoverlap = d_fftsize / winoverlap;
+        float num = static_cast<float>(winoverlap * len) / static_cast<float>(d_fftsize);
+        int nffts = static_cast<int>(ceilf(num));
+
+        // Clear this as we will be accumulating in the for loop over nffts
+        memset(d_pdu_magbuf, 0, sizeof(double)*d_fftsize);
+
+        size_t min = 0;
+        size_t max = std::min(d_fftsize, static_cast<int>(len));
+        for(int n = 0; n < nffts; n++) {
+          // Clear in case (max-min) < d_fftsize
+          memset(d_residbufs[d_nconnections], 0x00, sizeof(float)*d_fftsize);
+
+          // Copy in as much of the input samples as we can
+          memcpy(d_residbufs[d_nconnections], &in[min], sizeof(float)*(max-min));
+
+          // Apply the window and FFT; copy data into the PDU
+          // magnitude buffer.
+          fft(d_fbuf, d_residbufs[d_nconnections], d_fftsize);
+          for(int x = 0; x < d_fftsize; x++) {
+            d_pdu_magbuf[x] += (double)d_fbuf[x];
+          }
+
+          // Increment our indices; set max up to the number of
+          // samples in the input PDU.
+          min += fftoverlap;
+          max = std::min(max + fftoverlap, len);
+        }
+
+        // Perform the averaging
+        for(int x = 0; x < d_fftsize; x++) {
+          d_pdu_magbuf[x] /= static_cast<double>(nffts);
+        }
+
+        //update gui per-pdu
+        d_qApplication->postEvent(d_main_gui,
+                                  new FreqUpdateEvent(d_magbufs, d_fftsize));
+      }
+    }
+
   } /* namespace qtgui */
 } /* namespace gr */
