@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2012 Free Software Foundation, Inc.
+ * Copyright 2012,2015 Free Software Foundation, Inc.
  *
  * This file is part of GNU Radio
  *
@@ -49,6 +49,8 @@ namespace gr {
       int d_nconnections;
       int d_nrows;
 
+      const pmt::pmt_t d_port;
+
       bool d_shift;
       fft::fft_complex *d_fft;
 
@@ -133,6 +135,7 @@ namespace gr {
       void enable_menu(bool en);
       void enable_grid(bool en);
       void disable_legend();
+      void enable_axis_labels(bool en);
 
       int work(int noutput_items,
 	       gr_vector_const_void_star &input_items,
