@@ -29,8 +29,6 @@
 namespace gr {
   namespace uhd {
 
-    static const size_t ALL_CHANS = ::uhd::usrp::multi_usrp::ALL_CHANS;
-
     usrp_sink::sptr
     usrp_sink::make(const ::uhd::device_addr_t &device_addr,
 		    const ::uhd::io_type_t &io_type,
@@ -137,7 +135,17 @@ namespace gr {
       return _dev->set_tx_freq(tune_request, chan);
     }
 
-    SET_CENTER_FREQ_FROM_INTERNALS(usrp_sink_impl, set_tx_freq);
+    ::uhd::tune_result_t
+    usrp_sink_impl::_set_center_freq_from_internals(size_t chan, pmt::pmt_t direction)
+    {
+      _chans_to_tune.reset(chan);
+      if (pmt::eqv(direction, ant_direction_rx())) {
+        // TODO: what happens if the RX device is not instantiated? Catch error?
+        return _dev->set_rx_freq(_curr_tune_req[chan], _stream_args.channels[chan]);
+      } else {
+        return _dev->set_tx_freq(_curr_tune_req[chan], _stream_args.channels[chan]);
+      }
+    }
 
     double
     usrp_sink_impl::get_center_freq(size_t chan)
@@ -171,12 +179,16 @@ namespace gr {
 
     void usrp_sink_impl::set_normalized_gain(double norm_gain, size_t chan)
     {
+#ifdef UHD_USRP_MULTI_USRP_NORMALIZED_GAIN
+        _dev->set_normalized_tx_gain(norm_gain, chan);
+#else
       if (norm_gain > 1.0 || norm_gain < 0.0) {
         throw std::runtime_error("Normalized gain out of range, must be in [0, 1].");
       }
       ::uhd::gain_range_t gain_range = get_gain_range(chan);
       double abs_gain = (norm_gain * (gain_range.stop() - gain_range.start())) + gain_range.start();
       set_gain(abs_gain, chan);
+#endif
     }
 
     double
@@ -196,6 +208,9 @@ namespace gr {
     double
     usrp_sink_impl::get_normalized_gain(size_t chan)
     {
+#ifdef UHD_USRP_MULTI_USRP_NORMALIZED_GAIN
+        return _dev->get_normalized_tx_gain(chan);
+#else
       ::uhd::gain_range_t gain_range = get_gain_range(chan);
       double norm_gain =
         (get_gain(chan) - gain_range.start()) /
@@ -204,6 +219,7 @@ namespace gr {
       if (norm_gain > 1.0) return 1.0;
       if (norm_gain < 0.0) return 0.0;
       return norm_gain;
+#endif
     }
 
     std::vector<std::string>
@@ -321,7 +337,8 @@ namespace gr {
     {
       _update_stream_args(stream_args);
 #ifdef GR_UHD_USE_STREAM_API
-      _tx_stream.reset();
+      if(_tx_stream)
+        _tx_stream.reset();
 #else
       throw std::runtime_error("not implemented in this version");
 #endif
@@ -368,6 +385,7 @@ namespace gr {
         }
       }
 
+      boost::this_thread::disable_interruption disable_interrupt;
 #ifdef GR_UHD_USE_STREAM_API
       //send all ninput_items with metadata
       const size_t num_sent = _tx_stream->send
@@ -377,6 +395,7 @@ namespace gr {
         (input_items, ninput_items, _metadata,
          *_type, ::uhd::device::SEND_MODE_FULL_BUFF, 1.0);
 #endif
+      boost::this_thread::restore_interruption restore_interrupt(disable_interrupt);
 
       //if using length_tags, decrement items left to send by the number of samples sent
       if(not pmt::is_null(_length_tag_key) && _nitems_to_send > 0) {
@@ -499,14 +518,14 @@ namespace gr {
           // If it's on the first sample, immediately do the tune:
           GR_LOG_DEBUG(d_debug_logger, boost::format("Received tx_freq on start of burst."));
           pmt::pmt_t freq_cmd = pmt::make_dict();
-          freq_cmd = pmt::dict_add(freq_cmd, pmt::mp("freq"), value);
+          freq_cmd = pmt::dict_add(freq_cmd, cmd_freq_key(), value);
           msg_handler_command(freq_cmd);
         }
         else if(pmt::equal(key, FREQ_KEY)) {
           // If it's not on the first sample, queue this command and only tx until here:
           GR_LOG_DEBUG(d_debug_logger, boost::format("Received tx_freq mid-burst."));
           pmt::pmt_t freq_cmd = pmt::make_dict();
-          freq_cmd = pmt::dict_add(freq_cmd, pmt::mp("freq"), value);
+          freq_cmd = pmt::dict_add(freq_cmd, cmd_freq_key(), value);
           commands_in_burst.push_back(freq_cmd);
           max_count = my_tag_count + 1;
           in_burst_cmd_offset = my_tag_count;
@@ -571,7 +590,8 @@ namespace gr {
     usrp_sink_impl::start(void)
     {
 #ifdef GR_UHD_USE_STREAM_API
-      _tx_stream = _dev->get_tx_stream(_stream_args);
+      if (not _tx_stream)
+        _tx_stream = _dev->get_tx_stream(_stream_args);
 #endif
 
       _metadata.start_of_burst = true;
@@ -609,7 +629,8 @@ namespace gr {
       _nitems_to_send = 0;
 
 #ifdef GR_UHD_USE_STREAM_API
-      _tx_stream->send(gr_vector_const_void_star(_nchan), 0, _metadata, 1.0);
+      if(_tx_stream)
+        _tx_stream->send(gr_vector_const_void_star(_nchan), 0, _metadata, 1.0);
 #else
       _dev->get_device()->send
         (gr_vector_const_void_star(_nchan), 0, _metadata,
@@ -618,5 +639,18 @@ namespace gr {
       return true;
     }
 
+
+    void
+    usrp_sink_impl::setup_rpc()
+    {
+#ifdef GR_CTRLPORT
+      add_rpc_variable(
+        rpcbasic_sptr(new rpcbasic_register_handler<usrp_block>(
+          alias(), "command",
+          "", "UHD Commands",
+          RPC_PRIVLVL_MIN, DISPNULL)));
+#endif /* GR_CTRLPORT */
+    }
+
   } /* namespace uhd */
 } /* namespace gr */
