@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2010-2015 Free Software Foundation, Inc.
+ * Copyright 2010-2016 Free Software Foundation, Inc.
  *
  * This file is part of GNU Radio
  *
@@ -31,6 +31,7 @@
 namespace gr {
   namespace uhd {
 
+
     usrp_source::sptr
     usrp_source::make(const ::uhd::device_addr_t &device_addr,
                       const ::uhd::io_type_t &io_type,
@@ -53,30 +54,35 @@ namespace gr {
 
     usrp_source::sptr
     usrp_source::make(const ::uhd::device_addr_t &device_addr,
-                      const ::uhd::stream_args_t &stream_args)
+                      const ::uhd::stream_args_t &stream_args,
+                      const bool issue_stream_cmd_on_start)
     {
       check_abi();
       return usrp_source::sptr
-        (new usrp_source_impl(device_addr, stream_args_ensure(stream_args)));
+        (new usrp_source_impl(device_addr, stream_args_ensure(stream_args), issue_stream_cmd_on_start));
     }
 
     usrp_source_impl::usrp_source_impl(const ::uhd::device_addr_t &device_addr,
-                                       const ::uhd::stream_args_t &stream_args):
+                                       const ::uhd::stream_args_t &stream_args,
+                                       const bool issue_stream_cmd_on_start):
       usrp_block("gr uhd usrp source",
                     io_signature::make(0, 0, 0),
                     args_to_io_sig(stream_args)),
       usrp_block_impl(device_addr, stream_args, ""),
-      _tag_now(false)
+      _recv_timeout(0.1), // seconds
+      _recv_one_packet(true),
+      _tag_now(false),
+      _issue_stream_cmd_on_start(issue_stream_cmd_on_start)
     {
       std::stringstream str;
       str << name() << unique_id();
       _id = pmt::string_to_symbol(str.str());
 
       _samp_rate = this->get_samp_rate();
-      _center_freq = this->get_center_freq(0);
 #ifdef GR_UHD_USE_STREAM_API
       _samps_per_packet = 1;
 #endif
+      register_msg_cmd_handler(cmd_tag_key(), boost::bind(&usrp_source_impl::_cmd_handler_tag, this, _1));
     }
 
     usrp_source_impl::~usrp_source_impl()
@@ -137,15 +143,23 @@ namespace gr {
     usrp_source_impl::set_center_freq(const ::uhd::tune_request_t tune_request,
                                       size_t chan)
     {
-      const size_t user_chan = chan;
       chan = _stream_args.channels[chan];
       const ::uhd::tune_result_t res = _dev->set_rx_freq(tune_request, chan);
-      _center_freq = this->get_center_freq(user_chan);
       _tag_now = true;
       return res;
     }
 
-    SET_CENTER_FREQ_FROM_INTERNALS(usrp_source_impl, set_rx_freq);
+    ::uhd::tune_result_t
+    usrp_source_impl::_set_center_freq_from_internals(size_t chan, pmt::pmt_t direction)
+    {
+      _chans_to_tune.reset(chan);
+      if (pmt::eqv(direction, ant_direction_tx())) {
+        // TODO: what happens if the TX device is not instantiated? Catch error?
+        return _dev->set_tx_freq(_curr_tune_req[chan], _stream_args.channels[chan]);
+      } else {
+        return _dev->set_rx_freq(_curr_tune_req[chan], _stream_args.channels[chan]);
+      }
+    }
 
     double
     usrp_source_impl::get_center_freq(size_t chan)
@@ -177,12 +191,16 @@ namespace gr {
 
     void usrp_source_impl::set_normalized_gain(double norm_gain, size_t chan)
     {
+#ifdef UHD_USRP_MULTI_USRP_NORMALIZED_GAIN
+        _dev->set_normalized_rx_gain(norm_gain, chan);
+#else
       if (norm_gain > 1.0 || norm_gain < 0.0) {
         throw std::runtime_error("Normalized gain out of range, must be in [0, 1].");
       }
       ::uhd::gain_range_t gain_range = get_gain_range(chan);
       double abs_gain = (norm_gain * (gain_range.stop() - gain_range.start())) + gain_range.start();
       set_gain(abs_gain, chan);
+#endif
     }
 
     double
@@ -202,6 +220,9 @@ namespace gr {
     double
     usrp_source_impl::get_normalized_gain(size_t chan)
     {
+#ifdef UHD_USRP_MULTI_USRP_NORMALIZED_GAIN
+        return _dev->get_normalized_rx_gain(chan);
+#else
       ::uhd::gain_range_t gain_range = get_gain_range(chan);
       double norm_gain =
         (get_gain(chan) - gain_range.start()) /
@@ -210,6 +231,7 @@ namespace gr {
       if (norm_gain > 1.0) return 1.0;
       if (norm_gain < 0.0) return 0.0;
       return norm_gain;
+#endif
     }
 
     std::vector<std::string>
@@ -275,6 +297,104 @@ namespace gr {
         return _dev->get_rx_bandwidth_range(chan);
     }
 
+    std::vector<std::string>
+    usrp_source_impl::get_lo_names(size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->get_rx_lo_names(chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    const std::string
+    usrp_source_impl::get_lo_source(const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->get_rx_lo_source(name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    std::vector<std::string>
+    usrp_source_impl::get_lo_sources(const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->get_rx_lo_sources(name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    void
+    usrp_source_impl::set_lo_source(const std::string &src, const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->set_rx_lo_source(src, name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    bool
+    usrp_source_impl::get_lo_export_enabled(const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->get_rx_lo_export_enabled(name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    void
+    usrp_source_impl::set_lo_export_enabled(bool enabled, const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->set_rx_lo_export_enabled(enabled, name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    ::uhd::freq_range_t
+    usrp_source_impl::get_lo_freq_range(const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->get_rx_lo_freq_range(name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    double
+    usrp_source_impl::get_lo_freq(const std::string &name, size_t chan)
+    {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->get_rx_lo_freq(name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
+    double
+    usrp_source_impl::set_lo_freq(double freq, const std::string &name, size_t chan) {
+#ifdef UHD_USRP_MULTI_USRP_LO_CONFIG_API
+        chan = _stream_args.channels[chan];
+        return _dev->set_rx_lo_freq(freq, name, chan);
+#else
+        throw std::runtime_error("not implemented in this version");
+#endif
+    }
+
     void
     usrp_source_impl::set_auto_dc_offset(const bool enable, size_t chan)
     {
@@ -348,12 +468,19 @@ namespace gr {
     {
       _update_stream_args(stream_args);
 #ifdef GR_UHD_USE_STREAM_API
-      _rx_stream.reset();
+      if(_rx_stream)
+        _rx_stream.reset();
 #else
       throw std::runtime_error("not implemented in this version");
 #endif
     }
 
+    void
+    usrp_source_impl::_cmd_handler_tag(const pmt::pmt_t &tag)
+    {
+      _tag_now = true;
+    }
+
     void
     usrp_source_impl::set_start_time(const ::uhd::time_spec_t &time)
     {
@@ -369,6 +496,16 @@ namespace gr {
         {
             _dev->issue_stream_cmd(cmd, _stream_args.channels[i]);
         }
+        _tag_now = true;
+    }
+
+    void
+    usrp_source_impl::set_recv_timeout(
+            const double timeout,
+            const bool one_packet
+    ) {
+        _recv_timeout = timeout;
+        _recv_one_packet = one_packet;
     }
 
     bool
@@ -381,18 +518,20 @@ namespace gr {
         _samps_per_packet = _rx_stream->get_max_num_samps();
       }
 #endif
-      //setup a stream command that starts streaming slightly in the future
-      static const double reasonable_delay = 0.1; //order of magnitude over RTT
-      ::uhd::stream_cmd_t stream_cmd(::uhd::stream_cmd_t::STREAM_MODE_START_CONTINUOUS);
-      stream_cmd.stream_now = _stream_now;
-      if(_start_time_set) {
-        _start_time_set = false; //cleared for next run
-        stream_cmd.time_spec = _start_time;
+      if(_issue_stream_cmd_on_start){
+        //setup a stream command that starts streaming slightly in the future
+        static const double reasonable_delay = 0.1; //order of magnitude over RTT
+        ::uhd::stream_cmd_t stream_cmd(::uhd::stream_cmd_t::STREAM_MODE_START_CONTINUOUS);
+        stream_cmd.stream_now = _stream_now;
+        if(_start_time_set) {
+          _start_time_set = false; //cleared for next run
+          stream_cmd.time_spec = _start_time;
+        }
+        else {
+          stream_cmd.time_spec = get_time_now() + ::uhd::time_spec_t(reasonable_delay);
+        }
+        this->issue_stream_cmd(stream_cmd);
       }
-      else {
-        stream_cmd.time_spec = get_time_now() + ::uhd::time_spec_t(reasonable_delay);
-      }
-      this->issue_stream_cmd(stream_cmd);
       _tag_now = true;
       return true;
     }
@@ -409,7 +548,12 @@ namespace gr {
       while(true) {
 #ifdef GR_UHD_USE_STREAM_API
         const size_t bpi = ::uhd::convert::get_bytes_per_item(_stream_args.cpu_format);
-        _rx_stream->recv(outputs, nbytes/bpi, _metadata, 0.0);
+        if(_rx_stream)
+          // get the remaining samples out of the buffers
+          _rx_stream->recv(outputs, nbytes/bpi, _metadata, 0.0);
+        else
+          // no rx streamer -- nothing to flush
+          break;
 #else
         _dev->get_device()->recv
           (outputs, nbytes/_type->size, _metadata, *_type,
@@ -490,19 +634,18 @@ namespace gr {
                            gr_vector_void_star &output_items)
     {
       boost::recursive_mutex::scoped_lock lock(d_mutex);
+      boost::this_thread::disable_interruption disable_interrupt;
 #ifdef GR_UHD_USE_STREAM_API
       //In order to allow for low-latency:
       //We receive all available packets without timeout.
       //This call can timeout under regular operation...
-      size_t num_samps = _rx_stream->recv
-        (output_items, noutput_items, _metadata, 0.0);
-
-      //If receive resulted in a timeout condition:
-      //We now receive a single packet with a large timeout.
-      if(_metadata.error_code == ::uhd::rx_metadata_t::ERROR_CODE_TIMEOUT) {
-        num_samps = _rx_stream->recv
-          (output_items, noutput_items, _metadata, 0.1, true/*one pkt*/);
-      }
+      size_t num_samps = _rx_stream->recv(
+          output_items,
+          noutput_items,
+          _metadata,
+          _recv_timeout,
+          _recv_one_packet
+      );
 #else
       size_t num_samps = _dev->get_device()->recv
         (output_items, noutput_items, _metadata,
@@ -514,8 +657,8 @@ namespace gr {
            ::uhd::device::RECV_MODE_ONE_PACKET, 1.0);
       }
 #endif
-
-      //handle possible errors conditions
+      boost::this_thread::restore_interruption restore_interrupt(disable_interrupt);
+      // handle possible errors conditions
       switch(_metadata.error_code) {
       case ::uhd::rx_metadata_t::ERROR_CODE_NONE:
         if(_tag_now) {
@@ -530,7 +673,7 @@ namespace gr {
             this->add_item_tag(i, nitems_written(0), RATE_KEY,
                                pmt::from_double(_samp_rate), _id);
             this->add_item_tag(i, nitems_written(0), FREQ_KEY,
-                               pmt::from_double(_center_freq), _id);
+                               pmt::from_double(this->get_center_freq(i)), _id);
           }
         }
         break;
@@ -545,13 +688,25 @@ namespace gr {
         return work(noutput_items, input_items, output_items);
 
       default:
-        std::cout << boost::format("UHD source block got error code 0x%x")
-          % _metadata.error_code << std::endl;
+        //GR_LOG_WARN(d_logger, boost::format("USRP Source Block caught rx error: %d") % _metadata.strerror());
+        GR_LOG_WARN(d_logger, boost::format("USRP Source Block caught rx error code: %d") % _metadata.error_code);
         return num_samps;
       }
 
       return num_samps;
     }
 
+    void
+    usrp_source_impl::setup_rpc()
+    {
+#ifdef GR_CTRLPORT
+      add_rpc_variable(
+        rpcbasic_sptr(new rpcbasic_register_handler<usrp_block>(
+          alias(), "command",
+          "", "UHD Commands",
+          RPC_PRIVLVL_MIN, DISPNULL)));
+#endif /* GR_CTRLPORT */
+    }
+
   } /* namespace uhd */
 } /* namespace gr */
